@@ -797,6 +797,11 @@ Want to download more cities?</source>
         <source>Empty Now</source>
         <translation>Очистить</translation>
     </message>
+    <message>
+        <location line="+43"/>
+        <source>Restore default settings</source>
+        <translation>Настройки по умолчанию</translation>
+    </message>
 </context>
 <context>
     <name>TMapView</name>
