@@ -188,6 +188,13 @@ TCompleter::filterList(QList< QPair<QString,QColor> > list, QString filter)
    filtred << list.at(i);
    idFiltred << i;
   }
- return filtred;
+
+ filter=filter.section("*",0,0).toLower();
+ for(int i=0;i<filtred.count();i++)
+  if(filtred.at(i).first.left(filter.length()).toLower()==filter)
+  {
+   filtred.move(i,0);
+   idFiltred.move(i,0);
+  }
 }
 
