@@ -79,7 +79,8 @@ void TMapElement::paint(QPainter *p, const QStyleOptionGraphicsItem *option, QWi
       qreal w=map->params.routeWidth/qreal(1.7);
       tPath.addEllipse(edge->node[0]->pos(),w,w);
       tPath.addEllipse(edge->node[1]->pos(),w,w);
-      tPath.addPath(edge->path);
+      tPath.moveTo(edge->node[0]->pos());
+      tPath.lineTo(edge->node[1]->pos());
      }
 
     // black border for transfers
@@ -95,7 +96,7 @@ void TMapElement::paint(QPainter *p, const QStyleOptionGraphicsItem *option, QWi
      else continue;
 
      // black border for lines
-     p->setPen(QPen(Qt::black,width*qreal(1.3),Qt::SolidLine,Qt::FlatCap,Qt::RoundJoin));
+     p->setPen(QPen(Qt::black,width*qreal(1.4),Qt::SolidLine,Qt::FlatCap,Qt::RoundJoin));
      p->drawPath(edge->path);
      p->setPen(QPen(color,width,Qt::SolidLine,Qt::FlatCap,Qt::RoundJoin));
      p->drawPath(edge->path);
