@@ -11,6 +11,8 @@
 #include <QToolBox>
 #include <QFileDialog>
 #include <QTextStream>
+#include <QImageWriter>
+#include <QImageReader>
 
 #include <QWheelEvent>
 #include <math.h>
@@ -23,9 +25,11 @@
 #include <QQueue>
 #include <QPair>
 #include <QScrollBar>
+#include <QTouchEvent>
 
 #include "../zip/unzip.h"
 #include "../ini/inisettings.h"
+#include "../flickcharm.h"
 
 #include "mapview.h"
 #include "mapgraphics.h"
@@ -35,6 +39,8 @@
 #include "maplayer.h"
 #include "mapelement.h"
 
+
+
 //#if defined(Q_WS_MAEMO_5)
 //#include <QtGui/QX11Info>
 //#include <X11/Xlib.h>
@@ -52,19 +58,23 @@ QString ru="ru";
 TMapView::TMapView(QString AppPath, QString CachePath) :
   CachedMap(NULL),nodeBeginID(-1),nodeEndID(-1),
   nodeBegin(NULL),nodeEnd(NULL),nodeClick(NULL),
-  mapFade(NULL),mapWidth(0),mapHeight(0)
+  mapFade(NULL),nodePopup(NULL),mapWidth(0),mapHeight(0)
 {
  hide();
  setAttribute(Qt::WA_OpaquePaintEvent);
- setAttribute(Qt::WA_NoSystemBackground);
+ setAttribute(Qt::WA_AcceptTouchEvents);
+ viewport()->grabGesture(Qt::PinchGesture);
+ viewport()->setAttribute(Qt::WA_AcceptTouchEvents);
+ //setAttribute(Qt::WA_NoSystemBackground);
  //setAttribute(Qt::WA_PaintOnScreen);
-
 #if OPENGL_ENABLE //defined(Q_WS_MAEMO_5) //defined(Q_OS_WIN32) //!(defined Q_OS_WINCE || defined Q_OS_SYMBIAN)
  if(QGLFormat::hasOpenGL())
  {
-  QGLWidget *gl=new QGLWidget(QGLFormat(QGL::SampleBuffers));
-  //gl->context()->format().setDoubleBuffer(1);
-  setViewport(gl); //QGLWidget(QGLFormat::defaultFormat(), this)
+  QGLFormat fmt = QGLFormat::defaultFormat();
+  fmt.setDirectRendering(1);
+  fmt.setDoubleBuffer(1);
+  QGLWidget *gl=new QGLWidget(fmt);
+  setViewport(gl);
  }
 #endif
  
@@ -106,7 +116,7 @@ TMapView::TMapView(QString AppPath, QString CachePath) :
  setDragMode(QGraphicsView::ScrollHandDrag);
  setTransformationAnchor(AnchorUnderMouse);
  setResizeAnchor(AnchorViewCenter);
- 
+
  state.mapCached=0;
  state.mapDragging=0;
  params.delayIndex=params.delayMetro=0;
@@ -115,10 +125,10 @@ TMapView::TMapView(QString AppPath, QString CachePath) :
  params.appName=qApp->applicationName();
  codePage = QTextCodec::codecForName("windows-1251");
  
- TimerPopup.setInterval(500);
+ TimerPopup.setInterval(1000);
  TimerPopup.setSingleShot(1);
  connect(&TimerPopup, SIGNAL(timeout()), this, SLOT(popupByTimer()));
- TimerDragging.setInterval(800);
+ TimerDragging.setInterval(1000);
  TimerDragging.setSingleShot(1);
  connect(&TimerDragging, SIGNAL(timeout()), this, SLOT(mapStopDragging()));
  
@@ -154,7 +164,9 @@ void TMapView::mapStopDragging()
    if(config.enableAntialiasing)
    {
     setRenderHints(Antialiasing);
-    foreach(TMapNode *node,nodeByID) node->update(); // dont need in 4.7.2 ???
+    viewport()->update();
+    if(config.enableKineticScroll) fcharm->activateOn(this);
+    //foreach(TMapNode *node,nodeByID) node->update(); // dont need in 4.7.2 ???
    }
    //else
   }
@@ -183,7 +195,10 @@ void TMapView::keyPressEvent(QKeyEvent *event)
         scaleMap(1/qreal(1.2));
         break;
     case '*':
-        fitMap();
+        (config.MobileVersion ? zoomIn():fitMap());
+        break;
+    case '#':
+        zoomOut();
         break;
     default:
         QGraphicsView::keyPressEvent(event);
@@ -195,6 +210,7 @@ void TMapView::wheelEvent(QWheelEvent *event)
  if(event->buttons()==Qt::MidButton)
  {
   int offset=event->delta();
+  mapStartDragging();
   if (event->orientation() == Qt::Horizontal)
    horizontalScrollBar()->setValue(horizontalScrollBar()->value()-offset);
   else
@@ -205,39 +221,120 @@ void TMapView::wheelEvent(QWheelEvent *event)
 
 }
 
+
+//void drawBackgroundImage(QPainter* p,const QRectF& clip,QImage *bgimage)
+//{
+// p->eraseRect(clip);
+
+// QMatrix wm(p->worldMatrix());
+
+// if(wm.m11()!=1 && clip.intersects(bgimage->rect()))
+// {
+//  QImage img;
+//  QRectF c( clip );
+//  if( wm.m11() > 1 )
+//   c.adjust( -1, -1, 1, 1 );
+
+//  qreal cx = c.x();
+//  qreal cy = c.y();
+//  qreal cw = qMax( 16., c.width() );
+//  qreal ch = qMax( 16., c.height() );
+
+//  if( cx < 0 )
+//  {
+//   cw += cx;
+//   cx = 0;
+//  }
+//  if( cy < 0 )
+//  {
+//   ch += cy;
+//   cy = 0;
+//  }
+//  cw = qMin( cw, bgimage->width() - cx );
+//  ch = qMin( ch, bgimage->height() - cy );
+//  img = bgimage->copy( QRectF( cx, cy, cw, ch ).toRect() );
+//  cw = qRound( cw * wm.m11() );
+//  ch = qRound( ch * wm.m11() );
+//  img = img.scaled( (int) cw, (int) ch, Qt::IgnoreAspectRatio,
+//                    wm.m11() > .8 ? Qt::SmoothTransformation : Qt::FastTransformation );
+//  p->setWorldMatrix( QMatrix( 1, 0, 0, 1, wm.dx(), wm.dy() ) );
+//  p->drawImage( QPointF( cx * wm.m11(), cy * wm.m11() ), img );
+//  p->setWorldMatrix( wm );
+// }	else
+// {
+//  p->drawImage(
+//     QPointF( clip.x(), clip.y() ),
+//     *bgimage,
+//     QRectF( clip.x(), clip.y(),
+//             qMin( clip.width(), qreal( bgimage->width() ) ),
+//             qMin( clip.height(), qreal( bgimage->height() ) ) ) );
+// }
+//}
+
+
 void TMapView::drawBackground(QPainter *painter, const QRectF &rect)
 {
- Q_UNUSED(rect);
+// Q_UNUSED(rect);
 // testTimer.start();
 
   if(state.mapLoaded)
   {
    if(config.enableCaching)
    {
+
+    if(!mapWidth || !mapHeight) drawPaintCmd(painter,paintBkg);
+    if(!mapWidth || !mapHeight)
+    {
+     mapWidth=sceneRect().width();
+     mapHeight=sceneRect().height();
+    }
+    if(mapRect.isNull()) mapRect=QRect(-100,-100,mapWidth+200,mapHeight+200);
+
     if(state.mapCached)
-     painter->drawPixmap(0,0,*CachedMap);
+    {
+     painter->fillRect(mapRect,params.mapColorBack);
+//     drawBackgroundImage(painter,rect,CachedMap);
+     painter->drawPixmap(mapRect,*CachedMap);
+// painter->drawImage(QPointF(rect.x(),rect.y()),*CachedMap,QRectF(rect.x(),rect.y(),
+//                    qMin(rect.width(),qreal(CachedMap->width())),qMin(rect.height(),qreal(CachedMap->height()))));
+    }
     else
     {
-     if(!mapWidth || !mapHeight) drawPaintCmd(painter,paintBkg);
-     if(!mapWidth || !mapHeight)
-     {
-      mapWidth=sceneRect().width()-100;
-      mapHeight=sceneRect().height()-100;
-     }
-     QRect rect(0,0,mapWidth,mapHeight);
      if(CachedMap) delete CachedMap;
-     CachedMap = new QPixmap(mapWidth,mapHeight);
-     CachedMap->fill(params.mapColorBack);
-     if(QFile::exists(params.mapPath+".cache") && nodeByID.count())
+     CachedMap = new QPixmap(mapWidth+200,mapHeight+200);
+     //CachedMap = new QImage(mapWidth,mapHeight,QImage::Format_RGB555);
+     //CachedMap->fill(params.mapColorBack);
+     QString cacheImgPath = params.mapPath+".cache";
+     QFileInfo cacheInfo(cacheImgPath);
+     if(QFile::exists(cacheImgPath) && nodeByID.count())
      {
-      CachedMap->load(params.mapPath+".cache","PNG");
-      state.mapCached=1;
-      painter->drawPixmap(0,0,*CachedMap);
-      return;
+      if(cacheInfo.size()>1024)
+      {
+       QImage img(mapWidth,mapHeight,QImage::Format_RGB555);
+       QImageReader imgReader(cacheImgPath,"PNG");
+//       if(imgReader.canRead())
+//            QPixmap::fromImageReader(&imgReader);
+
+       //CachedMap->load(cacheImgPath,"PNG");
+       if(imgReader.read(&img))
+       {
+        state.mapCached=1;
+        painter->fillRect(mapRect,params.mapColorBack);
+//        drawBackgroundImage(painter,rect,CachedMap);
+        CachedMap->convertFromImage(img);
+        //painter->drawImage(QRect(0,0,mapWidth,mapHeight),img);
+        painter->drawPixmap(mapRect,*CachedMap);
+        return;
+       }
+      }else
+       QFile::remove(cacheImgPath);
      }
+     //QImage img(mapWidth,mapHeight,QImage::Format_RGB16);
      QPainter p(CachedMap);
      //p.scale(params.mapZoomFactor*1.2,params.mapZoomFactor*1.2);
+     p.translate(100,100);
      p.setRenderHints(Antialiasing);
+     p.fillRect(mapRect,QBrush(params.mapColorBack));
      if(config.enableDrawBackground) drawPaintCmd(&p,paintBkg);
      if(nodeByID.count())
      {
@@ -247,55 +344,36 @@ void TMapView::drawBackground(QPainter *painter, const QRectF &rect)
       if(mapFade) sv=mapFade->isVisible();
       if(sv) mapFade->setVisible(0);
       nodeBeginID=nodeEndID=-1;
-      foreach(TMapNode *node,nodeByID) node->update();
-      scene->render(&p,rect,rect);
+      //foreach(TMapNode *node,nodeByID) node->update();
+      scene->render(&p,mapRect,mapRect);
+      p.end();
       nodeBeginID=tmpBegin; nodeEndID=tmpEnd;
       if(sv) mapFade->setVisible(1);
      }
-//     if(!QFile::exists(params.mapPath+".cache"))
-//      CachedMap->save(params.mapPath+".cache","PNG");
      state.mapCached=1;
-     painter->drawPixmap(0,0,*CachedMap);
+     painter->fillRect(mapRect,params.mapColorBack);
+//     drawBackgroundImage(painter,rect,CachedMap);
+     painter->drawPixmap(mapRect,*CachedMap);
     }
    }else
    {
     if(!mapWidth || !mapHeight) drawPaintCmd(painter,paintBkg);
-    painter->fillRect(sceneRect(),params.mapColorBack);
+    painter->fillRect(rect,params.mapColorBack);
     if(config.enableDrawBackground) drawPaintCmd(painter,paintBkg);
     drawPaintCmd(painter,paintMap);
    }
 
   //qDebug() << testTimer.elapsed();
-
-
-//  // Debug scheme
-//  foreach(TMapRoute *route,layer.at(0)->route)
-//  {
-//   QColor color=route->color;
-//   color=color.darker();
-//   painter->setPen(QPen(color,2));
-//   foreach(TMapEdge *edge,route->edge)
-//   {
-//    if(!edge->addNode.count()) continue;
-//    //painter->drawText(edge->node[0]->pos(),QString("%1").arg(++num));
-//    foreach(QPoint point,edge->addNode) painter->drawPoint(point); //painter->drawText(point,QString("%1").arg(++num));
-//    painter->drawLine(edge->node[0]->pos(),edge->addNode[0]);
-//    painter->drawLine(edge->node[1]->pos(),edge->addNode[edge->addNode.count()-1]);
-//   }
-//  }
-
-  //state.mapLoaded=0;
  }
 // else
 // {
-//  if(layout()) layout().
-//  int size=64;
-//  QString text=" "+window()->windowTitle();
+//  int size=viewport()->rect().width()/2;
+//  QString text;//=" "+params.mapCityDisplay;
 //  QFont font("Tahoma");
 //  font.setPixelSize(size);
 //  QFontMetrics fm(font);
 //  int length=fm.width(text);
-//  QPointF point=rect.center()-QPoint(length/2-size,0);
+//  QPointF point=viewport()->rect().center()-QPoint(length/2-size,-size/2);
 //  painter->setFont(font);
 //  painter->drawPixmap(point-QPoint(size,size),window()->windowIcon().pixmap(size,size));
 //  painter->drawText(point,text);
@@ -309,12 +387,9 @@ void TMapView::scaleMap(qreal scaleFactor)
 {
  if(!state.mapLoaded) return;
  mapStartDragging();
- //if(config.enableAntialiasing) setRenderHints(NonAntialiasing);
  params.mapZoomFactor = matrix().scale(scaleFactor, scaleFactor).m11(); //mapRect(QRectF(0, 0, 1, 1)).width();
  if (params.mapZoomFactor < qreal(0.1) || params.mapZoomFactor > qreal(20)) return;
  scale(scaleFactor, scaleFactor);
- //state.mapCached=0;
- //if(config.enableAntialiasing) setRenderHints(Antialiasing);
 }
 
 void TMapView::zoomIn()
@@ -351,8 +426,8 @@ void TMapView::loadMap(QString fileName)
  if(!QFile::exists(fileName) || fileName.right(4).toLower()!=".pmz") return;
 
  QString imageFile,fname = fileName.section('/', -1 );
+
  // clear
- state.mapLoaded = 0;
  clearMap();
  //
 
@@ -368,11 +443,29 @@ void TMapView::loadMap(QString fileName)
  cty.beginGroup("Options");
  params.mapName=cty.value("Name");
  params.mapCityName=cty.value("CityName");
+ params.mapRusName=cty.value("RusName");
  QString Letter=codePage->fromUnicode(params.mapCityName.mid(0,1));
  if(BGN[ Letter[0] ]!="")
   config.enableTranslit=(config.locale!=ru);
+
+
+ // Set window title
+ QString CityName=params.mapCityName;
+ if(config.locale==ru && !params.mapRusName.isEmpty()) CityName=params.mapRusName;
+ else
+ {
+  Letter=codePage->fromUnicode(CityName.mid(0,1));
+  if(BGN[Letter[0]]!="") CityName=params.mapName;
+  Letter=codePage->fromUnicode(CityName.mid(0,1));
+  if(BGN[Letter[0]]!="") convertToTranslit(CityName);
+ }
+ //(params.countryName.isEmpty() ? "":" - ")+params.countryName+
+ if(WindowTitle.isEmpty()) WindowTitle=window()->windowTitle();
+ window()->setWindowTitle(CityName+(CityName.isEmpty() ? "":" - ")+WindowTitle);
+ qApp->processEvents();
+ params.mapCityDisplay=CityName;
+
  params.mapCountryName=cty.value("Country");
- params.mapRusName=cty.value("RusName");
  params.mapNeedVersion=cty.value("NeedVersion");
  params.mapAuthors=cty.value("MapAuthors");
  params.mapAuthors=convertToTextWithTags(params.mapAuthors);
@@ -425,11 +518,7 @@ void TMapView::loadMap(QString fileName)
  params.nodeFont.setStyleHint(QFont::Serif);
  params.nodeFont.setWeight(QFont::Bold);
  params.nodeFont.setFamily("Arial");
-#ifdef Q_OS_SYMBIAN
- params.nodeFont.setPointSize(6);
-#else
- params.nodeFont.setPointSize(8);
-#endif
+ params.nodeFont.setPixelSize(11); // 8pt
  params.nodeFontDefault=params.nodeFont;
 
  // load data
@@ -443,9 +532,7 @@ void TMapView::loadMap(QString fileName)
  addLayerToPaintCmd(layer.at(0),paintMap);
  scene->setItemIndexMethod(QGraphicsScene::BspTreeIndex);
 
- //loadInfo();
-
-
+ initNodeInfo();
 
  // fade screen
  QRectF rFade=sceneRect();
@@ -457,20 +544,8 @@ void TMapView::loadMap(QString fileName)
  mapFade->setVisible(0);
  scene->addItem(mapFade);
 
- QString CityName=params.mapCityName;
- if(config.locale=="ru" && !params.mapRusName.isEmpty()) CityName=params.mapRusName;
- else
- {
-  QString Letter=codePage->fromUnicode(CityName.mid(0,1));
-  if(BGN[Letter[0]]!="") CityName=params.mapName;
-  Letter=codePage->fromUnicode(CityName.mid(0,1));
-  if(BGN[Letter[0]]!="") convertToTranslit(CityName);
- }
- //(params.countryName.isEmpty() ? "":" - ")+params.countryName+
- if(WindowTitle.isEmpty()) WindowTitle=window()->windowTitle();
- window()->setWindowTitle(WindowTitle+(CityName.isEmpty() ? "":" - ")+CityName);
-
- state.mapLoaded = 1;
+ config.mapLastFile="";
+ state.mapLoaded=1;
 }
 
 void TMapView::exportMap(QString fileName,QString ext)
@@ -481,13 +556,15 @@ void TMapView::exportMap(QString fileName,QString ext)
   mapHeight=sceneRect().height()-100;
  }
  QRectF rect(0,0,mapWidth, mapHeight);
- QImage image(mapWidth, mapHeight, QImage::Format_ARGB32_Premultiplied);
+ QImage image(mapWidth, mapHeight, QImage::Format_RGB16);
  image.fill(0);
  QPainter painter(&image);
  painter.setRenderHints(Antialiasing);
  drawBackground(&painter,scene->sceneRect());
  scene->render(&painter,rect,rect);
- image.save(fileName,ext.toAscii());
+ QImageWriter imageWriter(fileName,ext.toAscii());
+ imageWriter.write(image);
+ //image.save(fileName,ext.toAscii());
 }
 
 bool TMapView::extractFiles(QString zipFile, QString extractFiles, QString extractDir)
@@ -880,7 +957,7 @@ void TMapView::drawPaintCmd(QPainter *p,QList<TPaintCmd> &paintCmd)
     QPen pen=p->pen();
     QBrush brush=p->brush();
     p->setPen(Qt::NoPen);
-    p->setBrush(QBrush(pen.color()));
+    p->setBrush(pen.color());
     p->drawPath(cmd.path);
     p->setPen(pen);
     p->setBrush(brush);
@@ -920,15 +997,21 @@ void TMapView::drawPaintCmd(QPainter *p,QList<TPaintCmd> &paintCmd)
 
 void TMapView::clearMap()
 {
+ state.mapLoaded=0;
+ state.mapCached=0;
+
  for(int i=0;i<paintBkg.count();i++)
    if(paintBkg.at(i).image!=NULL) delete paintBkg.at(i).image;
  paintBkg.clear();
  paintMap.clear();
- state.mapCached=0;
 
  for(int a=0;a<edgeCache.count();a++) delete edgeCache.at(a);
  edgeCache.clear();
 
+ if(CachedMap) delete CachedMap;
+ CachedMap = new QPixmap(0,0);
+ //CachedMap = new QImage(0,0,QImage::Format_Mono);
+
  mapDistance.clear();
  nodeBegin=NULL;
  nodeEnd=NULL;
@@ -937,7 +1020,7 @@ void TMapView::clearMap()
 
  scene->setSceneRect(QRect(0,0,0,0));
  scene->clear();
- mapWidth=0; mapHeight=0;
+ mapWidth=0; mapHeight=0; mapRect=QRect();
  params.nodeFont=QFont();
 
  for(int i=0;i<mapGraph.count();i++)
@@ -962,13 +1045,12 @@ void TMapView::clearMap()
   delete layer.at(i);
  }
  layer.clear();
-
 }
 
 void TMapView::loadMapLayers(QString LayerFilesTRP)
 {
  QStringList list,tprlist=LayerFilesTRP.split(",",QString::SkipEmptyParts);
- int nodeID=0;
+ int ind,nodeID=0;
  INISettings map(params.mapPath+"Metro.map",codePage);
  foreach(const QString &file, tprlist)
  {
@@ -1009,8 +1091,8 @@ void TMapView::loadMapLayers(QString LayerFilesTRP)
    cost=trp.valueToStringList("Delays");
    if(!cost.count())
    {
-    cost << trp.value("DelayDay");
-    cost << trp.value("DelayNight");
+    cost << trp.value("DelayDay","0");
+    cost << trp.value("DelayNight","0");
     cost << "0";
    }else cost << "0";
    foreach(const QString &str,cost) route->delay<<strTimeToSec(str);
@@ -1020,19 +1102,26 @@ void TMapView::loadMapLayers(QString LayerFilesTRP)
 
    // load nodes
    route->temp << str << trp.value("Driving");
+   QStringList nodeAliases2=trp.valueToStringList("Aliases2");
    trp.endGroup();
 
    str.remove(QRegExp("\\([^\\)]*\\)"));
-   QStringList nodesName=trp.toStringList(str);
-   foreach(const QString &nodeName,nodesName)
+   QStringList nodeNames=trp.toStringList(str);
+   ind=0;
+   foreach(const QString &nodeName,nodeNames)
    {
-    //qDebug() << nodeName;
     TMapNode *node = new TMapNode(this,route,nodeName,nodeID++);
+    if(nodeAliases2.count()>ind) node->Alias=nodeAliases2.at(ind);
+    //qDebug() << node->Name << node->Alias;
     route->node.append(node);
     nodeByID << node;
     scene->addItem(node);
+    ind++;
    }
-   nodesName.clear();
+   nodeNames.clear();
+   nodeAliases2.clear();
+
+   route->ID=mapLayer->route.count()+1;
    mapLayer->route.append(route);
 
 
@@ -1042,7 +1131,7 @@ void TMapView::loadMapLayers(QString LayerFilesTRP)
 
    for(int i=0;i<list.count();i=i+2) // Coordinates
    {
-    int ind=((i+2)/2)-1;
+    ind=((i+2)/2)-1;
     route->node.at(ind)->setPos(QPoint(list.at(i).toInt(),list.at(i+1).toInt()));
    }
    list.clear();
@@ -1073,7 +1162,8 @@ void TMapView::loadMapLayers(QString LayerFilesTRP)
     route->rect=QRect(list.at(0).toInt(),list.at(1).toInt(),list.at(2).toInt(),list.at(3).toInt());
 
    QString color=map.value("Color");
-   if(color.count()>2) route->color.setNamedColor("#"+color);
+   if(color.count()>2)
+        route->color.setNamedColor("#"+color);
    else route->color=QColor(0,0,0,0);
    color=map.value("LabelsColor");
    if(color.count()>2) route->labelsColor.setNamedColor("#"+color);
@@ -1084,6 +1174,19 @@ void TMapView::loadMapLayers(QString LayerFilesTRP)
 
    route->width=map.value("Width",LinesWidth).toFloat();
 
+   // color & styles of nodes (some memory economy)
+   route->ColorDark=route->color.darker();
+   route->EndPen=QPen(Qt::red,params.nodeWidth/qreal(2));
+   route->BeginPen=QPen(Qt::green,params.nodeWidth/qreal(2));
+   route->TransferPen=QPen(Qt::white,0);
+   route->TransferInfoPen=QPen(Qt::darkYellow,0);
+   route->NormalPen=QPen(route->color.lighter(),1);
+   route->NormalInfoPen=QPen(Qt::yellow,1);
+   route->ConstructPen=QPen(route->color,2);
+   route->ConstructInfoPen=QPen(Qt::darkYellow,2);
+
+
+
    map.endGroup();
    foreach(TMapNode *n,route->node) n->init();
    //route->active=1;
@@ -1211,11 +1314,12 @@ void TMapView::loadMapLayers(QString LayerFilesTRP)
      eCost=cost.value(i);
     }
     if(node2.isEmpty()) continue;
-    eCost.remove("("); //eCost.remove(")");
+    eCost.remove("(");
+    eCost.remove(")");//
 
     node1=map.toStringList(node1).at(0); // quoter chop
     node2=map.toStringList(node2).at(0);
-    //qDebug() << node1+" - "+node2+" = "+eCost;
+    //qDebug() << route->Name << node1+" - "+node2+" = "+eCost;
     //qDebug() << addEdge.count();
 
     edge = new TMapEdge(route,node1,node2,eCost);
@@ -1307,7 +1411,6 @@ void TMapView::addLayerToPaintCmd(TMapLayer *mapLayer,QList<TPaintCmd> &paintCmd
   QPainterPath pathRoute;
   QPen solid=QPen(route->color,route->width,Qt::SolidLine,Qt::SquareCap,Qt::RoundJoin);
   QPen dash=QPen(route->color,route->width*qreal(0.8),Qt::CustomDashLine,Qt::FlatCap,Qt::RoundJoin);
-
   dash.setDashPattern(dashPattern);
   cmd.pen=solid;
   cmd.type=dtPenColor;
@@ -1322,10 +1425,10 @@ void TMapView::addLayerToPaintCmd(TMapLayer *mapLayer,QList<TPaintCmd> &paintCmd
    polygon << edge->node[0]->pos();
    foreach(const QPointF &point,edge->addNode) polygon << point;
    polygon << edge->node[1]->pos();
-
+   //qDebug()<<polygon;
    if(polygon.count()>3) // because may be loop.
    {
-    // LOH-KOSTIL :)
+    // LOH.KO-STIL :)
     QGraphicsLineItem *lineItem1 = new QGraphicsLineItem(QLineF(polygon.first(),polygon.at(1))),
                       *lineItem2 = new QGraphicsLineItem(QLineF(polygon.last(), polygon.at(polygon.count()-2)));
     //lineItem1->setPen(QPen(Qt::black,1));//params.LinesWidth/2));
@@ -1367,7 +1470,8 @@ void TMapView::addLayerToPaintCmd(TMapLayer *mapLayer,QList<TPaintCmd> &paintCmd
      qreal w=tWidth/qreal(1.7);//tWidth*1.1;
      tPath.addEllipse(edge->node[i]->pos(),w,w);
     }
-    tPath.addPath(path);
+    tPath.moveTo(polygon.first());
+    tPath.lineTo(polygon.last());
     edge->path=path;
     mapGraph[edge->node[1]->ID][edge->node[0]->ID]->path=path;
     continue;
@@ -1442,7 +1546,8 @@ void TMapView::mouseMoveEvent(QMouseEvent *event)
 //  }
 //  state.mapDragging=1;
  }
- if(TimerPopup.isActive() && QLineF(nodeClickPos,event->pos()).length()>qreal(8.0)) TimerPopup.stop();
+ //TimerPopup.stop();
+ if(TimerPopup.isActive() && nodeClick && QLineF(nodeClickPos,event->pos()).length()>params.nodeWidth*5) TimerPopup.stop();
 }
 
 //void TMapView::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event)
@@ -1482,8 +1587,12 @@ void TMapView::mousePressEvent (QMouseEvent *event)
 
 void TMapView::popupByTimer()
 {
+ if(!nodeClick || !nodePopup) return;
  setDragMode(QGraphicsView::NoDrag);
- if(nodePopup) nodePopup->popup(mapToGlobal(nodeClickPos));
+ if(config.MobileVersion)
+  nodePopup->popup(QPoint(rect().width()/2-nodePopup->width()/2,rect().height()-nodePopup->height())+mapToGlobal(rect().topLeft()));
+ else
+  nodePopup->popup(mapToGlobal(nodeClickPos));
  if(config.enableLowqDrag && config.enableAntialiasing)
   setRenderHints(Antialiasing);
  setDragMode(QGraphicsView::ScrollHandDrag);
@@ -1493,11 +1602,9 @@ void TMapView::mouseReleaseEvent(QMouseEvent *event)
 {
  QGraphicsView::mouseReleaseEvent(event);
 
- emit nodeClicked(1);
-
  if(event->button()==Qt::LeftButton)
  {
-  TimerPopup.stop();
+
 //  if(state.mapDragging)
 //  {
 //   if(config.enableLowqDrag)
@@ -1513,8 +1620,14 @@ void TMapView::mouseReleaseEvent(QMouseEvent *event)
 //   state.mapDragging=0;
 //  }
  }
+//qDebug() << QLineF(nodeClickPos,event->pos()).length()<<params.nodeWidth*5;
+ if(QLineF(nodeClickPos,event->pos()).length()>params.nodeWidth*5)
+ {
+  nodeClick=NULL;
+  return;
+ }
 
- if(QLineF(nodeClickPos,event->pos()).length()>8) return;
+ TimerPopup.stop();
 
  if((event->button()==Qt::LeftButton || event->button()==Qt::RightButton)
     && (!nodeClick || nodeClick->type()!=elNode))
@@ -1537,6 +1650,7 @@ void TMapView::mouseReleaseEvent(QMouseEvent *event)
   if(nodeClick->TypeNode==TMapNode::ntConstruct) return;
 
  //testTimer.start();
+
   findTrack(nodeClick);
  //qDebug() << testTimer.elapsed();
  }
@@ -1547,7 +1661,7 @@ void TMapView::selectedNodesToFront(bool enable)
 {
  if(!selectedNode.count()) return;
 
- QRectF rect=selectedNode.at(0)->mapToScene(selectedNode.at(0)->boundingRect()).boundingRect();;
+ QRectF rect=selectedNode.at(0)->mapToScene(selectedNode.at(0)->boundingRect()).boundingRect();
  //qDebug() << rect;
  int z=(enable ? 2:-1),nPadding=params.nodeWidth*2;
  foreach(TMapNode *node, selectedNode)
@@ -1568,15 +1682,19 @@ void TMapView::selectedNodesToFront(bool enable)
  if(enable)
  {
   int wh=qMin(rect.width(),rect.height());
-  nPadding*=(wh/nPadding<qreal(4.0) ? 10:3);
+  nPadding*=(wh/nPadding<params.nodeWidth/2 ? params.nodeWidth:params.nodeWidth/3);
   rect.adjust(-nPadding,-nPadding,nPadding,nPadding);
-  //qDebug() << rect;
   fitInView(rect,Qt::KeepAspectRatio);
+  if(config.MobileVersion && selectedNode.count()==1)
+    verticalScrollBar()->setValue(verticalScrollBar()->value()+verticalScrollBar()->maximum()/12);
+
+  params.mapZoomFactor=matrix().m11();
  }
 }
 
 void TMapView::findTrack(TMapNode *begin,TMapNode *end)
 {
+ emit nodeClicked(1);
  if(!begin && end)
  {
   nodeEndID=end->ID;
@@ -1602,7 +1720,8 @@ void TMapView::findTrack(TMapNode *begin,TMapNode *end)
  nodeBeginID=nodeBegin->ID;
  if(nodeEnd) nodeEndID=nodeEnd->ID;
  if(!nodeEnd)mapDistance.clear();
- viewport()->repaint();
+ //viewport()->repaint();
+ qApp->processEvents();
  selectedNodesToFront(0);
 
  // calc distances
@@ -1618,11 +1737,11 @@ void TMapView::findTrack(TMapNode *begin,TMapNode *end)
     if(mapGraph[i][j]) workGraph[i][j]=mapGraph[i][j]->cost;
     else               workGraph[i][j]=0;
 
-    if(workGraph[i][j] && i==nodeBeginID)
+    if(!config.MobileVersion && workGraph[i][j] && i==nodeBeginID)
      workGraph[i][j]+=params.delayMetro*60+mapGraph[i][j]->node[1]->Route->delay.at(params.delayIndex);
 
     if(workGraph[i][j] && mapGraph[i][j]->type==TMapEdge::etTransfer)
-     workGraph[i][j]+=+mapGraph[i][j]->node[1]->Route->delay.at(params.delayIndex);
+     workGraph[i][j]+=mapGraph[i][j]->node[1]->Route->delay.at(params.delayIndex);
 
    }
   }
@@ -1632,11 +1751,17 @@ void TMapView::findTrack(TMapNode *begin,TMapNode *end)
   //   workGraph[nodeBeginID][i]+=param.delayMetro*60;
 
   dijkstra(nodeBeginID);
+
+  // caching calculations
+  foreach(TMapNode *node,nodeByID) node->Distance=QString::number(int(mapDistance[node->ID]/60));
+
   mapDistance[nodeBeginID]=params.delayMetro*60;
 
-  if(config.enableWriteTime) viewport()->repaint(); //foreach(TMapNode *node,nodeByID) node->update();
+  viewport()->update(); //foreach(TMapNode *node,nodeByID) node->update();
 
  }
+
+ emit nodeHasInfo(nodeBegin->HasInfo && nodeBegin && !nodeEnd);
  if(!nodeEnd) return;
  state.mapLoaded=0;
  // find track
@@ -1658,19 +1783,21 @@ void TMapView::findTrack(TMapNode *begin,TMapNode *end)
  for(int i=0;i<id.count()-1;i++)
   selectedEdge << mapGraph[id.at(i)][id.at(i+1)];
 
+ QString tf="HH:mm";
  QString br="<br>";
  QString line=" "+tr("line")+")";
- QString font="<table width=\"100%\" style=\"COLOR:white;BACKGROUND-COLOR:%1\"><tr><td align=\"center\"><b>&nbsp;%2</b></tr></table>";
+ QString font="<table width=\"100%\" style=\"COLOR:white;BACKGROUND-COLOR:%1\"><tr><td align=\"center\"><b>&nbsp;%2</b><td width=0 align=\"right\">%3</tr></table>";
  QString smallTag="<small>%1</small>";
 
  QString text=smallTag.arg("%4")+
-      font.arg("%5").arg("%6")+
+      font.arg("%5").arg("%6").arg(smallTag.arg("%8"))+
       "<center>"+smallTag.arg("%7")+"</center><hr>";
 
  text=text.arg(tr("Begin station:")) // 4
           .arg(nodeBegin->Route->color.name()) //5
           .arg(nodeBegin->DisplayName) //6
-          .arg("("+nodeBegin->Route->DisplayName+line); // 7
+          .arg("("+nodeBegin->Route->DisplayName+line) //7
+          .arg(QTime::currentTime().addSecs(config.MobileVersion ? 0 : params.delayMetro*60+nodeBegin->Route->delay.at(params.delayIndex)).toString(tf)); // 8
 
  int countTransfer=0;
  for(int i=selectedEdge.count()-1;i>=0;i--)
@@ -1680,14 +1807,21 @@ void TMapView::findTrack(TMapNode *begin,TMapNode *end)
   {
    countTransfer++;
    text+=smallTag.arg(tr("Go to station:"))+
-         font.arg(edge->node[1]->Route->color.name()).arg(edge->node[1]->DisplayName)+
+         font.arg(edge->node[1]->Route->color.name())
+             .arg(edge->node[1]->DisplayName)
+             .arg(smallTag.arg(QTime::currentTime().addSecs(mapDistance[edge->node[1]->ID]).toString(tf)))+
          smallTag.arg(tr("Transfer to station:"))+
-         font.arg(edge->node[0]->Route->color.name()).arg(edge->node[0]->DisplayName)+"<hr>";
+         font.arg(edge->node[0]->Route->color.name())
+             .arg(edge->node[0]->DisplayName)
+             .arg(smallTag.arg(QTime::currentTime().addSecs(mapDistance[edge->node[0]->ID]).toString(tf)))+"<hr>";
   }
  }
 
  text+=smallTag.arg(tr("Final station:"))+
-       font.arg(nodeEnd->Route->color.name()).arg(nodeEnd->DisplayName)+"<center>"+smallTag.arg("("+nodeEnd->Route->DisplayName+line)+"</center>";
+       font.arg(nodeEnd->Route->color.name())
+           .arg(nodeEnd->DisplayName)
+           .arg(smallTag.arg(QTime::currentTime().addSecs(mapDistance[nodeEndID]).toString(tf)))+
+           "<center>"+smallTag.arg("("+nodeEnd->Route->DisplayName+line)+"</center>";
 
  QString info;
  if(config.MobileVersion)
@@ -1732,7 +1866,9 @@ void TMapView::clearTrack()
  selectedEdge.clear();
  //if(nodeByID.count()) foreach(TMapNode *node,nodeByID) node->update();
  emit trackText("");
- if(config.MobileVersion) emit trackFinded(0);
+ emit nodeClicked(0);
+ emit nodeHasInfo(0);
+ if(config.MobileVersion) emit trackFinded(0); // hide LabelInfo
  viewport()->repaint();
 }
 
@@ -1758,22 +1894,26 @@ void TMapView::dijkstra(int nodeBeginID)
 
  mapDistance[nodeBeginID] = 0;
  queue.push_back(QPair <int,int> (mapDistance[nodeBeginID], nodeBeginID));
-
  while(!queue.empty())
  {
   qtmp = queue.first();
   queue.pop_front();
   i = qtmp.second;
+
   if (!visit[i])
   {
    visit[i] = true;
    for (j = 0; j<count; j++)
-   if (!visit[j] && workGraph[i][j] && workGraph[i][j]>0 && mapDistance[i]+workGraph[i][j] < mapDistance[j])
-   {
-    mapDistance[j] = mapDistance[i] + workGraph[i][j];
-    mapParentNode[j] = i;
-    queue.push_back(QPair <int,int>(-mapDistance[j], j));
-   }
+    if (!visit[j] && workGraph[i][j] && workGraph[i][j]>0)
+    {
+     qreal curDistance=mapDistance[i] + workGraph[i][j];
+     if(curDistance < mapDistance[j])
+     {
+      mapDistance[j] = curDistance;
+      mapParentNode[j] = i;
+      queue.push_back(QPair <int,int>(-curDistance, j));
+     }
+    }
   }
  }
 }
@@ -1793,7 +1933,40 @@ QList< QPair<QString,QColor> > TMapView::getNamesAndColors()
 
 void TMapView::defaultConfig()
 {
- config.mapLastFile=params.appPath+"map/Moscow.pmz";
+ QLocale::Country country=QLocale::system().country();
+ QString city,mapDir="map/";
+ city="Moscow";
+
+ if(country==QLocale::Ukraine) city="Kiev"; else
+ if(country==QLocale::Canada) city="Montreal"; else
+ if(country==QLocale::UnitedStates) city="New-York"; else
+ if(country==QLocale::Czech) city="Praha"; else
+ if(country==QLocale::Spanish) city="Madrid"; else
+ if(country==QLocale::Mexico) city="Mexico"; else
+ if(country==QLocale::France) city="Paris"; else
+ if(country==QLocale::Belgium) city="Brussels"; else
+ if(country==QLocale::Germany) city="Berlin"; else
+ if(country==QLocale::Austria) city="Wienn"; else
+ if(country==QLocale::UnitedKingdom) city="London"; else
+ if(country==QLocale::Italy) city="Milan"; else
+ if(country==QLocale::Finland) city="Helsinki"; else
+ if(country==QLocale::Brazil) city="San-Paulo"; else
+ if(country==QLocale::Thailand) city="Bangkok"; else
+ if(country==QLocale::India) city="Delhi"; else
+ if(country==QLocale::China) city="Beijing"; else
+ if(country==QLocale::HongKong) city="Hong-Kong"; else
+ if(country==QLocale::Taiwan) city="Taipei"; else
+ if(country==QLocale::RepublicOfKorea) city="Seoul"; else
+ if(country==QLocale::Japan) city="Tokyo";
+
+ city=config.mapLastFile=params.appPath+mapDir+city+".pmz";
+ //qDebug() << config.locale << QLocale::system().name() << config.mapLastFile;
+
+ if(!QFile::exists(config.mapLastFile))
+ {
+  QDir Dir=QDir(params.appPath+mapDir,"*.pmz",QDir::Name,QDir::Files);
+  city=Dir.entryList().first();
+ }
  config.enableCustomFont=0;
  config.enableLowqDrag=1;
  config.enableShowScrollBar=0;
@@ -1804,6 +1977,7 @@ void TMapView::defaultConfig()
  config.enableDrawBackground=1;
  config.enableCaching=config.MobileVersion;
  config.valueTransparency=60;
+ config.enableKineticScroll=0;
  params.mapColorBack=Qt::white;
 }
 
@@ -1820,10 +1994,9 @@ void TMapView::applyConfig()
   setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
  }
 
-
-  if(config.enableAntialiasing)
-       params.nodeFont.setStyleStrategy(QFont::PreferAntialias);
-  else params.nodeFont.setStyleStrategy(QFont::PreferDefault);
+ if(config.enableAntialiasing)
+      params.nodeFont.setStyleStrategy(QFont::PreferAntialias);
+ else params.nodeFont.setStyleStrategy(QFont::PreferDefault);
  //params.nodeFont.setStyleStrategy(QFont::ForceOutline);
 
  if(config.enableCaching)
@@ -1838,12 +2011,20 @@ void TMapView::applyConfig()
   state.mapCached=0;
  }
 
- // its last operation - load map with config
- if(nodeByID.isEmpty() && !config.mapLastFile.isEmpty()) loadMap(config.mapLastFile);
- config.mapLastFile="";
+ if(config.enableKineticScroll)
+ {
+  fcharm=new FlickCharm(this);
+  fcharm->activateOn(this);
+  connect(fcharm,SIGNAL(started()),this,SLOT(mapStartDragging()));
+  connect(fcharm,SIGNAL(finished()),this,SLOT(mapStopDragging()));
+ }
 
- //viewport()->update();
- foreach(TMapNode *node,nodeByID) node->update();
+
+ // its last operation - load last map on app start
+ if(params.mapFile.isEmpty() && !config.mapLastFile.isEmpty()) loadMap(config.mapLastFile);
+
+ viewport()->update();
+ //foreach(TMapNode *node,nodeByID) node->update();
 }
 
 void TMapView::setDelayMetro(int delay)
@@ -1899,10 +2080,54 @@ QString TMapView::convertToTextWithTags(QString text)
 
  return TagText;
 }
+
+void TMapView::initNodeInfo()
+{
+ QString HasInfoFileName=".info";
+ QSettings inf(params.mapPath+HasInfoFileName,QSettings::IniFormat);
+ if(QFile::exists(params.mapPath+HasInfoFileName))
+ {
+  foreach(TMapRoute *route,layer.first()->route)
+  {
+   inf.beginGroup(QString::number(route->ID));
+   foreach(TMapNode *node,route->node)
+     node->HasInfo=inf.value(QString::number(node->ID),"0").toBool();
+
+   inf.endGroup();
+  }
+  return;
+ }
+ QDir Dir=QDir(params.mapPath,"*.txt",QDir::Name,QDir::Files);
+ QStringList infoList=Dir.entryList();
+ foreach(TMapRoute *route,layer.first()->route)
+ {
+  int CountInfo=0;
+  inf.beginGroup(QString::number(route->ID));
+  foreach(const QString &infoFile,infoList)
+  {
+   INISettings Info(params.mapPath+infoFile,codePage);
+   if(!Info.value("Options/AddToInfo",0).toInt()) continue;
+
+   QString StrInfo;
+   foreach(TMapNode *node,route->node)
+   {
+    if(node->HasInfo) continue;
+    StrInfo=Info.value(QString("%1/%2").arg(route->Name).arg(node->Name));
+    if(StrInfo.length()<5) continue;
+    CountInfo++;
+    node->HasInfo=1;
+    inf.setValue(QString::number(node->ID),"1");
+    if(CountInfo==route->node.count()) break;
+   }
+   if(CountInfo==route->node.count()) break;
+  }
+  inf.endGroup();
+ }
+}
+
 void TMapView::loadNodeInfo(TMapNode *node)
 {
  if(node->Info.count()) return;
-
  QDir Dir=QDir(params.mapPath,"*.txt",QDir::Name,QDir::Files);
  QStringList infoList=Dir.entryList();
  foreach(const QString &infoFile,infoList)
@@ -1942,17 +2167,22 @@ void TMapView::setNodeEnd(TMapNode *Node)
 
 void TMapView::saveToFile()
 {
- QFileDialog saveDialog;
- QString filter,fileName=saveDialog.getSaveFileName(this,
- tr("Save file"), params.appPath,
- "JPG (*.jpg);;PNG (*.png);;TIF (*.tif);;BMP (*.bmp);;PPM (*.ppm);;XBM (*.xbm);;XPM (*.xpm)",&filter);
+ if(!QImageWriter::supportedImageFormats().count()) return;
+
+ QString supportFormats;
+ foreach(QString ext,QImageWriter::supportedImageFormats())
+  supportFormats+=QString("%1 (*.%2);;").arg(ext.toUpper()).arg(ext);
+\
+ QString filter=supportFormats.left(supportFormats.indexOf(";;")),
+         fileName=QFileDialog::getSaveFileName(0, tr("Save file"), params.appPath,
+                                               supportFormats,&filter);
  if(fileName.isEmpty()) return;
 
 //#ifndef QT_NO_CURSOR
 // QApplication::setOverrideCursor(Qt::WaitCursor);
 //#endif
 
- QString ext = filter.left(3);
+ QString ext = filter.left(4).trimmed();
  exportMap(fileName,ext);
 
 //#ifndef QT_NO_CURSOR
@@ -2036,6 +2266,7 @@ void TMapView::convertToTranslit(QString &text)
 {
  int max=text.count()-1;
  QString str=codePage->fromUnicode(text);
+ //qDebug() << str << text;
  for(int i=max;i>=0;i--)
  {
   QString translit=BGN[ str[i] ];
@@ -2050,4 +2281,67 @@ void TMapView::findedNodesToFront(QList<int> NodeID)
  selectedNode.clear();
  foreach(int ID,NodeID) selectedNode << nodeByID[ID];
  selectedNodesToFront(1);
+ emit nodeClicked(1);
+}
+
+bool TMapView::viewportEvent(QEvent *event)
+{
+ switch (event->type()) {
+ case QEvent::TouchBegin:
+ {
+// case QEvent::TouchUpdate:
+// case QEvent::TouchEnd:
+// {
+//  QTouchEvent *touchEvent = static_cast<QTouchEvent *>(event);
+//  QList<QTouchEvent::TouchPoint> touchPoints = touchEvent->touchPoints();
+//  if(touchPoints.count() == 2)
+//  {
+//   const QTouchEvent::TouchPoint &touchPoint0 = touchPoints.first();
+//   const QTouchEvent::TouchPoint &touchPoint1 = touchPoints.last();
+//   qreal currentScaleFactor=
+//         QLineF(touchPoint0.pos(),touchPoint1.pos()).length() /
+//         QLineF(touchPoint0.startPos(), touchPoint1.startPos()).length();
+//   if(touchEvent->touchPointStates() & Qt::TouchPointReleased)
+//   {
+//    params.mapZoomFactor *= currentScaleFactor;
+//    currentScaleFactor = 1;
+//   }
+//   setTransform(QTransform().scale(params.mapZoomFactor * currentScaleFactor,params.mapZoomFactor * currentScaleFactor));
+//   nodeClick=NULL;
+//  }
+  return true;
+ }
+ case QEvent::Gesture:
+  return gestureEvent(static_cast<QGestureEvent*>(event));
+ break;
+
+ default:
+  return QGraphicsView::viewportEvent(event);
+ }
+}
+
+bool TMapView::gestureEvent(QGestureEvent *event)
+{
+ if (QGesture *pinch = event->gesture(Qt::PinchGesture)){
+  //if(config.enableKineticScroll) fcharm->deactivateFrom(this);
+  pinchTriggered(static_cast<QPinchGesture *>(pinch));
+  //if(config.enableKineticScroll) fcharm->activateOn(this);
+ }
+ return true;
+}
+
+void TMapView::pinchTriggered(QPinchGesture *gesture)
+{
+ QPinchGesture::ChangeFlags changeFlags = gesture->changeFlags();
+ if (changeFlags & QPinchGesture::ScaleFactorChanged)
+ {
+//  qreal value = gesture->scaleFactor();
+//  qreal zoom = value*zoomFactor();
+
+//  if(zoom < 20 && zoom > 0.1) setZoomFactor(zoom);
+
+  scaleMap(gesture->scaleFactor());
+
+  nodeClick=NULL;
+ }
 }
